@@ -1,6 +1,8 @@
 """Support for EnOcean switches."""
 from __future__ import annotations
 
+from typing import Any
+
 from enocean.utils import combine_hex
 import voluptuous as vol
 
@@ -94,7 +96,7 @@ class EnOceanSwitch(EnOceanEntity, SwitchEntity):
         """Return the device name."""
         return self.dev_name
 
-    def turn_on(self, **kwargs):
+    def turn_on(self, **kwargs: Any) -> None:
         """Turn on the switch."""
         optional = [0x03]
         optional.extend(self.dev_id)
@@ -106,7 +108,7 @@ class EnOceanSwitch(EnOceanEntity, SwitchEntity):
         )
         self._on_state = True
 
-    def turn_off(self, **kwargs):
+    def turn_off(self, **kwargs: Any) -> None:
         """Turn off the switch."""
         optional = [0x03]
         optional.extend(self.dev_id)
