@@ -20,11 +20,10 @@ from homeassistant.components.fan import (
     SERVICE_SET_PERCENTAGE,
     SERVICE_TURN_OFF,
     SERVICE_TURN_ON,
-    SUPPORT_DIRECTION,
-    SUPPORT_OSCILLATE,
-    SUPPORT_SET_SPEED,
     FanEntity,
+    FanEntityFeature,
 )
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     ATTR_ASSUMED_STATE,
     ATTR_ENTITY_ID,
@@ -33,9 +32,11 @@ from homeassistant.const import (
     CONF_NAME,
     CONF_UNIQUE_ID,
     STATE_ON,
+    STATE_UNAVAILABLE,
+    STATE_UNKNOWN,
 )
 from homeassistant.core import Event, HomeAssistant, State, callback
-import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers import config_validation as cv, entity_registry as er
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.event import async_track_state_change_event
 from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
@@ -48,10 +49,16 @@ from .util import (
     states_equal,
 )
 
-SUPPORTED_FLAGS = {SUPPORT_SET_SPEED, SUPPORT_DIRECTION, SUPPORT_OSCILLATE}
+SUPPORTED_FLAGS = {
+    FanEntityFeature.SET_SPEED,
+    FanEntityFeature.DIRECTION,
+    FanEntityFeature.OSCILLATE,
+}
 
 DEFAULT_NAME = "Fan Group"
 
+# No limit on parallel updates to enable a group calling another group
+PARALLEL_UPDATES = 0
 
 PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
     {
@@ -70,15 +77,30 @@ async def async_setup_platform(
     async_add_entities: AddEntitiesCallback,
     discovery_info: DiscoveryInfoType | None = None,
 ) -> None:
-    """Set up the Group Cover platform."""
+    """Set up the Fan Group platform."""
     async_add_entities(
         [FanGroup(config.get(CONF_UNIQUE_ID), config[CONF_NAME], config[CONF_ENTITIES])]
     )
 
 
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
+    """Initialize Fan Group config entry."""
+    registry = er.async_get(hass)
+    entities = er.async_validate_entity_ids(
+        registry, config_entry.options[CONF_ENTITIES]
+    )
+
+    async_add_entities([FanGroup(config_entry.entry_id, config_entry.title, entities)])
+
+
 class FanGroup(GroupEntity, FanEntity):
     """Representation of a FanGroup."""
 
+    _attr_available: bool = False
     _attr_assumed_state: bool = True
 
     def __init__(self, unique_id: str | None, name: str, entities: list[str]) -> None:
@@ -90,7 +112,7 @@ class FanGroup(GroupEntity, FanEntity):
         self._direction = None
         self._supported_features = 0
         self._speed_count = 100
-        self._is_on = False
+        self._is_on: bool | None = False
         self._attr_name = name
         self._attr_extra_state_attributes = {ATTR_ENTITY_ID: entities}
         self._attr_unique_id = unique_id
@@ -106,7 +128,7 @@ class FanGroup(GroupEntity, FanEntity):
         return self._speed_count
 
     @property
-    def is_on(self) -> bool:
+    def is_on(self) -> bool | None:
         """Return true if the entity is on."""
         return self._is_on
 
@@ -174,24 +196,29 @@ class FanGroup(GroupEntity, FanEntity):
         if percentage == 0:
             await self.async_turn_off()
         await self._async_call_supported_entities(
-            SERVICE_SET_PERCENTAGE, SUPPORT_SET_SPEED, {ATTR_PERCENTAGE: percentage}
+            SERVICE_SET_PERCENTAGE,
+            FanEntityFeature.SET_SPEED,
+            {ATTR_PERCENTAGE: percentage},
         )
 
     async def async_oscillate(self, oscillating: bool) -> None:
         """Oscillate the fan."""
         await self._async_call_supported_entities(
-            SERVICE_OSCILLATE, SUPPORT_OSCILLATE, {ATTR_OSCILLATING: oscillating}
+            SERVICE_OSCILLATE,
+            FanEntityFeature.OSCILLATE,
+            {ATTR_OSCILLATING: oscillating},
         )
 
     async def async_set_direction(self, direction: str) -> None:
         """Set the direction of the fan."""
         await self._async_call_supported_entities(
-            SERVICE_SET_DIRECTION, SUPPORT_DIRECTION, {ATTR_DIRECTION: direction}
+            SERVICE_SET_DIRECTION,
+            FanEntityFeature.DIRECTION,
+            {ATTR_DIRECTION: direction},
         )
 
     async def async_turn_on(
         self,
-        speed: str | None = None,
         percentage: int | None = None,
         preset_mode: str | None = None,
         **kwargs: Any,
@@ -246,13 +273,29 @@ class FanGroup(GroupEntity, FanEntity):
         """Update state and attributes."""
         self._attr_assumed_state = False
 
-        on_states: list[State] = list(
-            filter(None, [self.hass.states.get(x) for x in self._entities])
-        )
-        self._is_on = any(state.state == STATE_ON for state in on_states)
-        self._attr_assumed_state |= not states_equal(on_states)
+        states = [
+            state
+            for entity_id in self._entities
+            if (state := self.hass.states.get(entity_id)) is not None
+        ]
+        self._attr_assumed_state |= not states_equal(states)
 
-        percentage_states = self._async_states_by_support_flag(SUPPORT_SET_SPEED)
+        # Set group as unavailable if all members are unavailable or missing
+        self._attr_available = any(state.state != STATE_UNAVAILABLE for state in states)
+
+        valid_state = any(
+            state.state not in (STATE_UNKNOWN, STATE_UNAVAILABLE) for state in states
+        )
+        if not valid_state:
+            # Set as unknown if all members are unknown or unavailable
+            self._is_on = None
+        else:
+            # Set as ON if any member is ON
+            self._is_on = any(state.state == STATE_ON for state in states)
+
+        percentage_states = self._async_states_by_support_flag(
+            FanEntityFeature.SET_SPEED
+        )
         self._percentage = reduce_attribute(percentage_states, ATTR_PERCENTAGE)
         self._attr_assumed_state |= not attribute_equal(
             percentage_states, ATTR_PERCENTAGE
@@ -270,13 +313,15 @@ class FanGroup(GroupEntity, FanEntity):
             self._speed_count = 100
 
         self._set_attr_most_frequent(
-            "_oscillating", SUPPORT_OSCILLATE, ATTR_OSCILLATING
+            "_oscillating", FanEntityFeature.OSCILLATE, ATTR_OSCILLATING
+        )
+        self._set_attr_most_frequent(
+            "_direction", FanEntityFeature.DIRECTION, ATTR_DIRECTION
         )
-        self._set_attr_most_frequent("_direction", SUPPORT_DIRECTION, ATTR_DIRECTION)
 
         self._supported_features = reduce(
             ior, [feature for feature in SUPPORTED_FLAGS if self._fans[feature]], 0
         )
         self._attr_assumed_state |= any(
-            state.attributes.get(ATTR_ASSUMED_STATE) for state in on_states
+            state.attributes.get(ATTR_ASSUMED_STATE) for state in states
         )
