@@ -8,12 +8,22 @@ import async_timeout
 from herepy import NoRouteFoundError, RouteMode, RoutingApi, RoutingResponse
 import voluptuous as vol
 
-from homeassistant.const import ATTR_ATTRIBUTION, CONF_UNIT_SYSTEM_IMPERIAL, Platform
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import (
+    ATTR_ATTRIBUTION,
+    CONF_API_KEY,
+    CONF_MODE,
+    CONF_UNIT_SYSTEM,
+    LENGTH_METERS,
+    LENGTH_MILES,
+    Platform,
+)
 from homeassistant.core import HomeAssistant
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.location import find_coordinates
 from homeassistant.helpers.update_coordinator import DataUpdateCoordinator, UpdateFailed
 from homeassistant.util import dt
+from homeassistant.util.unit_conversion import DistanceConverter
 
 from .const import (
     ATTR_DESTINATION,
@@ -23,9 +33,18 @@ from .const import (
     ATTR_DURATION_IN_TRAFFIC,
     ATTR_ORIGIN,
     ATTR_ORIGIN_NAME,
-    ATTR_ROUTE,
+    CONF_ARRIVAL_TIME,
+    CONF_DEPARTURE_TIME,
+    CONF_DESTINATION_ENTITY_ID,
+    CONF_DESTINATION_LATITUDE,
+    CONF_DESTINATION_LONGITUDE,
+    CONF_ORIGIN_ENTITY_ID,
+    CONF_ORIGIN_LATITUDE,
+    CONF_ORIGIN_LONGITUDE,
+    CONF_ROUTE_MODE,
     DEFAULT_SCAN_INTERVAL,
     DOMAIN,
+    IMPERIAL_UNITS,
     NO_ROUTE_ERROR_MESSAGE,
     TRAFFIC_MODE_ENABLED,
     TRAVEL_MODES_VEHICLE,
@@ -37,6 +56,58 @@ PLATFORMS = [Platform.SENSOR]
 _LOGGER = logging.getLogger(__name__)
 
 
+async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool:
+    """Set up HERE Travel Time from a config entry."""
+    api_key = config_entry.data[CONF_API_KEY]
+    here_client = RoutingApi(api_key)
+
+    arrival = (
+        dt.parse_time(config_entry.options[CONF_ARRIVAL_TIME])
+        if config_entry.options[CONF_ARRIVAL_TIME] is not None
+        else None
+    )
+    departure = (
+        dt.parse_time(config_entry.options[CONF_DEPARTURE_TIME])
+        if config_entry.options[CONF_DEPARTURE_TIME] is not None
+        else None
+    )
+
+    here_travel_time_config = HERETravelTimeConfig(
+        destination_latitude=config_entry.data.get(CONF_DESTINATION_LATITUDE),
+        destination_longitude=config_entry.data.get(CONF_DESTINATION_LONGITUDE),
+        destination_entity_id=config_entry.data.get(CONF_DESTINATION_ENTITY_ID),
+        origin_latitude=config_entry.data.get(CONF_ORIGIN_LATITUDE),
+        origin_longitude=config_entry.data.get(CONF_ORIGIN_LONGITUDE),
+        origin_entity_id=config_entry.data.get(CONF_ORIGIN_ENTITY_ID),
+        travel_mode=config_entry.data[CONF_MODE],
+        route_mode=config_entry.options[CONF_ROUTE_MODE],
+        units=config_entry.options[CONF_UNIT_SYSTEM],
+        arrival=arrival,
+        departure=departure,
+    )
+
+    coordinator = HereTravelTimeDataUpdateCoordinator(
+        hass,
+        here_client,
+        here_travel_time_config,
+    )
+    hass.data.setdefault(DOMAIN, {})[config_entry.entry_id] = coordinator
+    await hass.config_entries.async_forward_entry_setups(config_entry, PLATFORMS)
+
+    return True
+
+
+async def async_unload_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool:
+    """Unload a config entry."""
+    unload_ok = await hass.config_entries.async_unload_platforms(
+        config_entry, PLATFORMS
+    )
+    if unload_ok:
+        hass.data[DOMAIN].pop(config_entry.entry_id)
+
+    return unload_ok
+
+
 class HereTravelTimeDataUpdateCoordinator(DataUpdateCoordinator):
     """HERETravelTime DataUpdateCoordinator."""
 
@@ -107,19 +178,20 @@ class HereTravelTimeDataUpdateCoordinator(DataUpdateCoordinator):
             traffic_time: float = summary["baseTime"]
             if self.config.travel_mode in TRAVEL_MODES_VEHICLE:
                 traffic_time = summary["trafficTime"]
-            if self.config.units == CONF_UNIT_SYSTEM_IMPERIAL:
+            if self.config.units == IMPERIAL_UNITS:
                 # Convert to miles.
-                distance = distance / 1609.344
+                distance = DistanceConverter.convert(
+                    distance, LENGTH_METERS, LENGTH_MILES
+                )
             else:
                 # Convert to kilometers
                 distance = distance / 1000
             return HERERoutingData(
                 {
                     ATTR_ATTRIBUTION: attribution,
-                    ATTR_DURATION: summary["baseTime"] / 60,  # type: ignore[misc]
-                    ATTR_DURATION_IN_TRAFFIC: traffic_time / 60,
+                    ATTR_DURATION: round(summary["baseTime"] / 60),  # type: ignore[misc]
+                    ATTR_DURATION_IN_TRAFFIC: round(traffic_time / 60),
                     ATTR_DISTANCE: distance,
-                    ATTR_ROUTE: response.route_short,
                     ATTR_ORIGIN: ",".join(origin),
                     ATTR_DESTINATION: ",".join(destination),
                     ATTR_ORIGIN_NAME: waypoint[0]["mappedRoadName"],
@@ -135,33 +207,40 @@ class HereTravelTimeDataUpdateCoordinator(DataUpdateCoordinator):
     ) -> tuple[list[str], list[str], str | None, str | None]:
         """Prepare parameters for the HERE api."""
 
-        if self.config.origin_entity_id is not None:
-            origin = find_coordinates(self.hass, self.config.origin_entity_id)
-        else:
-            origin = self.config.origin
+        def _from_entity_id(entity_id: str) -> list[str]:
+            coordinates = find_coordinates(self.hass, entity_id)
+            if coordinates is None:
+                raise InvalidCoordinatesException(
+                    f"No coordinatnes found for {entity_id}"
+                )
+            try:
+                here_formatted_coordinates = coordinates.split(",")
+                vol.Schema(cv.gps(here_formatted_coordinates))
+            except (AttributeError, vol.Invalid) as ex:
+                raise InvalidCoordinatesException(
+                    f"{coordinates} are not valid coordinates"
+                ) from ex
+            return here_formatted_coordinates
 
+        # Destination
         if self.config.destination_entity_id is not None:
-            destination = find_coordinates(self.hass, self.config.destination_entity_id)
+            destination = _from_entity_id(self.config.destination_entity_id)
         else:
-            destination = self.config.destination
-        if destination is None:
-            raise InvalidCoordinatesException("Destination must be configured")
-        try:
-            here_formatted_destination = destination.split(",")
-            vol.Schema(cv.gps(here_formatted_destination))
-        except (vol.Invalid) as ex:
-            raise InvalidCoordinatesException(
-                f"{destination} are not valid coordinates"
-            ) from ex
-        if origin is None:
-            raise InvalidCoordinatesException("Origin must be configured")
-        try:
-            here_formatted_origin = origin.split(",")
-            vol.Schema(cv.gps(here_formatted_origin))
-        except (AttributeError, vol.Invalid) as ex:
-            raise InvalidCoordinatesException(
-                f"{origin} are not valid coordinates"
-            ) from ex
+            destination = [
+                str(self.config.destination_latitude),
+                str(self.config.destination_longitude),
+            ]
+
+        # Origin
+        if self.config.origin_entity_id is not None:
+            origin = _from_entity_id(self.config.origin_entity_id)
+        else:
+            origin = [
+                str(self.config.origin_latitude),
+                str(self.config.origin_longitude),
+            ]
+
+        # Arrival/Departure
         arrival: str | None = None
         departure: str | None = None
         if self.config.arrival is not None:
@@ -172,7 +251,7 @@ class HereTravelTimeDataUpdateCoordinator(DataUpdateCoordinator):
         if arrival is None and departure is None:
             departure = "now"
 
-        return (here_formatted_origin, here_formatted_destination, arrival, departure)
+        return (origin, destination, arrival, departure)
 
 
 def build_hass_attribution(source_attribution: dict) -> str | None:
