@@ -3,12 +3,13 @@ from __future__ import annotations
 
 from typing import Any
 
-from homeassistant.components.device_tracker import SOURCE_TYPE_GPS
+from homeassistant.components.device_tracker import SourceType
 from homeassistant.components.device_tracker.config_entry import TrackerEntity
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.core import HomeAssistant, callback
+from homeassistant.core import CALLBACK_TYPE, HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
 from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from .account import IcloudAccount, IcloudDevice
 from .const import (
@@ -19,16 +20,12 @@ from .const import (
 )
 
 
-async def async_setup_scanner(hass: HomeAssistant, config, see, discovery_info=None):
-    """Old way of setting up the iCloud tracker."""
-
-
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities
+    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
 ) -> None:
     """Set up device tracker for iCloud component."""
-    account = hass.data[DOMAIN][entry.unique_id]
-    tracked = set()
+    account: IcloudAccount = hass.data[DOMAIN][entry.unique_id]
+    tracked = set[str]()
 
     @callback
     def update_account():
@@ -43,7 +40,7 @@ async def async_setup_entry(
 
 
 @callback
-def add_entities(account, async_add_entities, tracked):
+def add_entities(account: IcloudAccount, async_add_entities, tracked):
     """Add new tracker entities from the account."""
     new_tracked = []
 
@@ -54,8 +51,7 @@ def add_entities(account, async_add_entities, tracked):
         new_tracked.append(IcloudTrackerEntity(account, device))
         tracked.add(dev_id)
 
-    if new_tracked:
-        async_add_entities(new_tracked, True)
+    async_add_entities(new_tracked, True)
 
 
 class IcloudTrackerEntity(TrackerEntity):
@@ -65,7 +61,7 @@ class IcloudTrackerEntity(TrackerEntity):
         """Set up the iCloud tracker entity."""
         self._account = account
         self._device = device
-        self._unsub_dispatcher = None
+        self._unsub_dispatcher: CALLBACK_TYPE | None = None
 
     @property
     def unique_id(self) -> str:
@@ -93,14 +89,14 @@ class IcloudTrackerEntity(TrackerEntity):
         return self._device.location[DEVICE_LOCATION_LONGITUDE]
 
     @property
-    def battery_level(self) -> int:
+    def battery_level(self) -> int | None:
         """Return the battery level of the device."""
         return self._device.battery_level
 
     @property
-    def source_type(self) -> str:
+    def source_type(self) -> SourceType:
         """Return the source type, eg gps or router, of the device."""
-        return SOURCE_TYPE_GPS
+        return SourceType.GPS
 
     @property
     def icon(self) -> str:
@@ -115,32 +111,34 @@ class IcloudTrackerEntity(TrackerEntity):
     @property
     def device_info(self) -> DeviceInfo:
         """Return the device information."""
-        return {
-            "identifiers": {(DOMAIN, self._device.unique_id)},
-            "name": self._device.name,
-            "manufacturer": "Apple",
-            "model": self._device.device_model,
-        }
+        return DeviceInfo(
+            configuration_url="https://icloud.com/",
+            identifiers={(DOMAIN, self._device.unique_id)},
+            manufacturer="Apple",
+            model=self._device.device_model,
+            name=self._device.name,
+        )
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Register state update callback."""
         self._unsub_dispatcher = async_dispatcher_connect(
             self.hass, self._account.signal_device_update, self.async_write_ha_state
         )
 
-    async def async_will_remove_from_hass(self):
+    async def async_will_remove_from_hass(self) -> None:
         """Clean up after entity before removal."""
-        self._unsub_dispatcher()
+        if self._unsub_dispatcher:
+            self._unsub_dispatcher()
 
 
 def icon_for_icloud_device(icloud_device: IcloudDevice) -> str:
-    """Return a battery icon valid identifier."""
+    """Return an icon for the device."""
     switcher = {
-        "iPad": "mdi:tablet-ipad",
-        "iPhone": "mdi:cellphone-iphone",
+        "iPad": "mdi:tablet",
+        "iPhone": "mdi:cellphone",
         "iPod": "mdi:ipod",
-        "iMac": "mdi:desktop-mac",
-        "MacBookPro": "mdi:laptop-mac",
+        "iMac": "mdi:monitor",
+        "MacBookPro": "mdi:laptop",
     }
 
     return switcher.get(icloud_device.device_class, "mdi:cellphone-link")
