@@ -2,26 +2,18 @@
 from __future__ import annotations
 
 from collections.abc import Callable
-from datetime import datetime
 from typing import Any
 
 from xknx import XKNX
-from xknx.devices import Cover as XknxCover, Device as XknxDevice
+from xknx.devices import Cover as XknxCover
 
 from homeassistant import config_entries
 from homeassistant.components.cover import (
     ATTR_POSITION,
     ATTR_TILT_POSITION,
-    DEVICE_CLASS_BLIND,
-    SUPPORT_CLOSE,
-    SUPPORT_CLOSE_TILT,
-    SUPPORT_OPEN,
-    SUPPORT_OPEN_TILT,
-    SUPPORT_SET_POSITION,
-    SUPPORT_SET_TILT_POSITION,
-    SUPPORT_STOP,
-    SUPPORT_STOP_TILT,
+    CoverDeviceClass,
     CoverEntity,
+    CoverEntityFeature,
 )
 from homeassistant.const import (
     CONF_DEVICE_CLASS,
@@ -29,9 +21,8 @@ from homeassistant.const import (
     CONF_NAME,
     Platform,
 )
-from homeassistant.core import HomeAssistant, callback
+from homeassistant.core import HomeAssistant
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-from homeassistant.helpers.event import async_track_utc_time_change
 from homeassistant.helpers.typing import ConfigType
 
 from .const import DATA_KNX_CONFIG, DOMAIN
@@ -75,39 +66,43 @@ class KNXCover(KnxEntity, CoverEntity):
                 group_address_position=config.get(CoverSchema.CONF_POSITION_ADDRESS),
                 travel_time_down=config[CoverSchema.CONF_TRAVELLING_TIME_DOWN],
                 travel_time_up=config[CoverSchema.CONF_TRAVELLING_TIME_UP],
+                invert_updown=config[CoverSchema.CONF_INVERT_UPDOWN],
                 invert_position=config[CoverSchema.CONF_INVERT_POSITION],
                 invert_angle=config[CoverSchema.CONF_INVERT_ANGLE],
             )
         )
         self._unsubscribe_auto_updater: Callable[[], None] | None = None
+
         self._attr_entity_category = config.get(CONF_ENTITY_CATEGORY)
+        _supports_tilt = False
+        self._attr_supported_features = (
+            CoverEntityFeature.CLOSE
+            | CoverEntityFeature.OPEN
+            | CoverEntityFeature.SET_POSITION
+        )
+        if self._device.step.writable:
+            _supports_tilt = True
+            self._attr_supported_features |= (
+                CoverEntityFeature.CLOSE_TILT
+                | CoverEntityFeature.OPEN_TILT
+                | CoverEntityFeature.STOP_TILT
+            )
+        if self._device.supports_angle:
+            _supports_tilt = True
+            self._attr_supported_features |= CoverEntityFeature.SET_TILT_POSITION
+        if self._device.supports_stop:
+            self._attr_supported_features |= CoverEntityFeature.STOP
+            if _supports_tilt:
+                self._attr_supported_features |= CoverEntityFeature.STOP_TILT
 
         self._attr_device_class = config.get(CONF_DEVICE_CLASS) or (
-            DEVICE_CLASS_BLIND if self._device.supports_angle else None
+            CoverDeviceClass.BLIND if _supports_tilt else None
         )
-        self._attr_supported_features = (
-            SUPPORT_CLOSE | SUPPORT_OPEN | SUPPORT_SET_POSITION
-        )
-        if self._device.supports_stop:
-            self._attr_supported_features |= SUPPORT_STOP | SUPPORT_STOP_TILT
-        if self._device.supports_angle:
-            self._attr_supported_features |= SUPPORT_SET_TILT_POSITION
-        if self._device.step.writable:
-            self._attr_supported_features |= (
-                SUPPORT_CLOSE_TILT | SUPPORT_OPEN_TILT | SUPPORT_STOP_TILT
-            )
         self._attr_unique_id = (
             f"{self._device.updown.group_address}_"
             f"{self._device.position_target.group_address}"
         )
 
-    @callback
-    async def after_update_callback(self, device: XknxDevice) -> None:
-        """Call after device was updated."""
-        self.async_write_ha_state()
-        if self._device.is_traveling():
-            self.start_auto_updater()
-
     @property
     def current_cover_position(self) -> int | None:
         """Return the current position of the cover.
@@ -115,8 +110,9 @@ class KNXCover(KnxEntity, CoverEntity):
         None is unknown, 0 is closed, 100 is fully open.
         """
         # In KNX 0 is open, 100 is closed.
-        pos = self._device.current_position()
-        return 100 - pos if pos is not None else None
+        if (pos := self._device.current_position()) is not None:
+            return 100 - pos
+        return None
 
     @property
     def is_closed(self) -> bool | None:
@@ -152,15 +148,13 @@ class KNXCover(KnxEntity, CoverEntity):
     async def async_stop_cover(self, **kwargs: Any) -> None:
         """Stop the cover."""
         await self._device.stop()
-        self.stop_auto_updater()
 
     @property
     def current_cover_tilt_position(self) -> int | None:
         """Return current tilt position of cover."""
-        if not self._device.supports_angle:
-            return None
-        ang = self._device.current_angle()
-        return 100 - ang if ang is not None else None
+        if (angle := self._device.current_angle()) is not None:
+            return 100 - angle
+        return None
 
     async def async_set_cover_tilt_position(self, **kwargs: Any) -> None:
         """Move the cover tilt to a specific position."""
@@ -178,25 +172,3 @@ class KNXCover(KnxEntity, CoverEntity):
     async def async_stop_cover_tilt(self, **kwargs: Any) -> None:
         """Stop the cover tilt."""
         await self._device.stop()
-        self.stop_auto_updater()
-
-    def start_auto_updater(self) -> None:
-        """Start the autoupdater to update Home Assistant while cover is moving."""
-        if self._unsubscribe_auto_updater is None:
-            self._unsubscribe_auto_updater = async_track_utc_time_change(
-                self.hass, self.auto_updater_hook
-            )
-
-    def stop_auto_updater(self) -> None:
-        """Stop the autoupdater."""
-        if self._unsubscribe_auto_updater is not None:
-            self._unsubscribe_auto_updater()
-            self._unsubscribe_auto_updater = None
-
-    @callback
-    def auto_updater_hook(self, now: datetime) -> None:
-        """Call for the autoupdater."""
-        self.async_write_ha_state()
-        if self._device.position_reached():
-            self.hass.async_create_task(self._device.auto_stop_if_necessary())
-            self.stop_auto_updater()
