@@ -1,16 +1,15 @@
 """Support for Ruckus Unleashed devices."""
 from __future__ import annotations
 
-from homeassistant.components.device_tracker import SOURCE_TYPE_ROUTER
+from homeassistant.components.device_tracker import SourceType
 from homeassistant.components.device_tracker.config_entry import ScannerEntity
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers import entity_registry
-from homeassistant.helpers.device_registry import CONNECTION_NETWORK_MAC
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 
 from .const import (
-    API_ACCESS_POINT,
     API_CLIENTS,
     API_NAME,
     COORDINATOR,
@@ -21,12 +20,12 @@ from .const import (
 
 
 async def async_setup_entry(
-    hass: HomeAssistant, entry: ConfigEntry, async_add_entities
+    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
 ) -> None:
     """Set up device tracker for Ruckus Unleashed component."""
     coordinator = hass.data[DOMAIN][entry.entry_id][COORDINATOR]
 
-    tracked = set()
+    tracked: set[str] = set()
 
     @callback
     def router_update():
@@ -39,7 +38,7 @@ async def async_setup_entry(
         coordinator.async_add_listener(router_update)
     )
 
-    registry = await entity_registry.async_get_registry(hass)
+    registry = entity_registry.async_get(hass)
     restore_entities(registry, coordinator, entry, async_add_entities, tracked)
 
 
@@ -56,8 +55,7 @@ def add_new_entities(coordinator, async_add_entities, tracked):
         new_tracked.append(RuckusUnleashedDevice(coordinator, mac, device[API_NAME]))
         tracked.add(mac)
 
-    if new_tracked:
-        async_add_entities(new_tracked)
+    async_add_entities(new_tracked)
 
 
 @callback
@@ -78,8 +76,7 @@ def restore_entities(registry, coordinator, entry, async_add_entities, tracked):
             )
             tracked.add(entity.unique_id)
 
-    if missing:
-        async_add_entities(missing)
+    async_add_entities(missing)
 
 
 class RuckusUnleashedDevice(CoordinatorEntity, ScannerEntity):
@@ -92,8 +89,8 @@ class RuckusUnleashedDevice(CoordinatorEntity, ScannerEntity):
         self._name = name
 
     @property
-    def unique_id(self) -> str:
-        """Return a unique ID."""
+    def mac_address(self) -> str:
+        """Return a mac address."""
         return self._mac
 
     @property
@@ -112,20 +109,6 @@ class RuckusUnleashedDevice(CoordinatorEntity, ScannerEntity):
         return self._mac in self.coordinator.data[API_CLIENTS]
 
     @property
-    def source_type(self) -> str:
+    def source_type(self) -> SourceType:
         """Return the source type."""
-        return SOURCE_TYPE_ROUTER
-
-    @property
-    def device_info(self) -> dict | None:
-        """Return the device information."""
-        if self.is_connected:
-            return {
-                "name": self.name,
-                "connections": {(CONNECTION_NETWORK_MAC, self._mac)},
-                "via_device": (
-                    CONNECTION_NETWORK_MAC,
-                    self.coordinator.data[API_CLIENTS][self._mac][API_ACCESS_POINT],
-                ),
-            }
-        return None
+        return SourceType.ROUTER
