@@ -2,15 +2,8 @@
 from datetime import timedelta
 import logging
 
-from homeassistant.components.sensor import STATE_CLASS_MEASUREMENT
-from homeassistant.const import (
-    DEVICE_CLASS_ENERGY,
-    DEVICE_CLASS_POWER,
-    ENERGY_WATT_HOUR,
-    PERCENTAGE,
-    POWER_WATT,
-)
-from homeassistant.util import dt as dt_util
+from homeassistant.components.sensor import SensorDeviceClass, SensorStateClass
+from homeassistant.const import ENERGY_WATT_HOUR, PERCENTAGE, POWER_WATT
 
 from .models import SolarEdgeSensorEntityDescription
 
@@ -40,10 +33,9 @@ SENSOR_TYPES = [
         json_key="lifeTimeData",
         name="Lifetime energy",
         icon="mdi:solar-power",
-        last_reset=dt_util.utc_from_timestamp(0),
-        state_class=STATE_CLASS_MEASUREMENT,
-        unit_of_measurement=ENERGY_WATT_HOUR,
-        device_class=DEVICE_CLASS_ENERGY,
+        state_class=SensorStateClass.TOTAL,
+        native_unit_of_measurement=ENERGY_WATT_HOUR,
+        device_class=SensorDeviceClass.ENERGY,
     ),
     SolarEdgeSensorEntityDescription(
         key="energy_this_year",
@@ -51,8 +43,8 @@ SENSOR_TYPES = [
         name="Energy this year",
         entity_registry_enabled_default=False,
         icon="mdi:solar-power",
-        unit_of_measurement=ENERGY_WATT_HOUR,
-        device_class=DEVICE_CLASS_ENERGY,
+        native_unit_of_measurement=ENERGY_WATT_HOUR,
+        device_class=SensorDeviceClass.ENERGY,
     ),
     SolarEdgeSensorEntityDescription(
         key="energy_this_month",
@@ -60,8 +52,8 @@ SENSOR_TYPES = [
         name="Energy this month",
         entity_registry_enabled_default=False,
         icon="mdi:solar-power",
-        unit_of_measurement=ENERGY_WATT_HOUR,
-        device_class=DEVICE_CLASS_ENERGY,
+        native_unit_of_measurement=ENERGY_WATT_HOUR,
+        device_class=SensorDeviceClass.ENERGY,
     ),
     SolarEdgeSensorEntityDescription(
         key="energy_today",
@@ -69,20 +61,21 @@ SENSOR_TYPES = [
         name="Energy today",
         entity_registry_enabled_default=False,
         icon="mdi:solar-power",
-        unit_of_measurement=ENERGY_WATT_HOUR,
-        device_class=DEVICE_CLASS_ENERGY,
+        native_unit_of_measurement=ENERGY_WATT_HOUR,
+        device_class=SensorDeviceClass.ENERGY,
     ),
     SolarEdgeSensorEntityDescription(
         key="current_power",
         json_key="currentPower",
         name="Current Power",
         icon="mdi:solar-power",
-        state_class=STATE_CLASS_MEASUREMENT,
-        unit_of_measurement=POWER_WATT,
-        device_class=DEVICE_CLASS_POWER,
+        state_class=SensorStateClass.MEASUREMENT,
+        native_unit_of_measurement=POWER_WATT,
+        device_class=SensorDeviceClass.POWER,
     ),
     SolarEdgeSensorEntityDescription(
         key="site_details",
+        json_key="status",
         name="Site details",
         entity_registry_enabled_default=False,
     ),
@@ -145,46 +138,56 @@ SENSOR_TYPES = [
         icon="mdi:car-battery",
     ),
     SolarEdgeSensorEntityDescription(
-        key="purchased_power",
+        key="purchased_energy",
         json_key="Purchased",
-        name="Imported Power",
+        name="Imported Energy",
         entity_registry_enabled_default=False,
-        icon="mdi:flash",
+        state_class=SensorStateClass.TOTAL_INCREASING,
+        native_unit_of_measurement=ENERGY_WATT_HOUR,
+        device_class=SensorDeviceClass.ENERGY,
     ),
     SolarEdgeSensorEntityDescription(
-        key="production_power",
+        key="production_energy",
         json_key="Production",
-        name="Production Power",
+        name="Production Energy",
         entity_registry_enabled_default=False,
-        icon="mdi:flash",
+        state_class=SensorStateClass.TOTAL_INCREASING,
+        native_unit_of_measurement=ENERGY_WATT_HOUR,
+        device_class=SensorDeviceClass.ENERGY,
     ),
     SolarEdgeSensorEntityDescription(
-        key="consumption_power",
+        key="consumption_energy",
         json_key="Consumption",
-        name="Consumption Power",
+        name="Consumption Energy",
         entity_registry_enabled_default=False,
-        icon="mdi:flash",
+        state_class=SensorStateClass.TOTAL_INCREASING,
+        native_unit_of_measurement=ENERGY_WATT_HOUR,
+        device_class=SensorDeviceClass.ENERGY,
     ),
     SolarEdgeSensorEntityDescription(
-        key="selfconsumption_power",
+        key="selfconsumption_energy",
         json_key="SelfConsumption",
-        name="SelfConsumption Power",
+        name="SelfConsumption Energy",
         entity_registry_enabled_default=False,
-        icon="mdi:flash",
+        state_class=SensorStateClass.TOTAL_INCREASING,
+        native_unit_of_measurement=ENERGY_WATT_HOUR,
+        device_class=SensorDeviceClass.ENERGY,
     ),
     SolarEdgeSensorEntityDescription(
-        key="feedin_power",
+        key="feedin_energy",
         json_key="FeedIn",
-        name="Exported Power",
+        name="Exported Energy",
         entity_registry_enabled_default=False,
-        icon="mdi:flash",
+        state_class=SensorStateClass.TOTAL_INCREASING,
+        native_unit_of_measurement=ENERGY_WATT_HOUR,
+        device_class=SensorDeviceClass.ENERGY,
     ),
     SolarEdgeSensorEntityDescription(
         key="storage_level",
         json_key="STORAGE",
         name="Storage Level",
         entity_registry_enabled_default=False,
-        state_class=STATE_CLASS_MEASUREMENT,
-        unit_of_measurement=PERCENTAGE,
+        state_class=SensorStateClass.MEASUREMENT,
+        native_unit_of_measurement=PERCENTAGE,
     ),
 ]
