@@ -1,55 +1,29 @@
 """Implementation of the musiccast media player."""
 from __future__ import annotations
 
+import contextlib
 import logging
+from typing import Any
 
 from aiomusiccast import MusicCastGroupException, MusicCastMediaContent
 from aiomusiccast.features import ZoneFeature
-import voluptuous as vol
 
+from homeassistant.components import media_source
 from homeassistant.components.media_player import (
-    PLATFORM_SCHEMA,
     BrowseMedia,
+    MediaClass,
     MediaPlayerEntity,
+    MediaPlayerEntityFeature,
+    MediaPlayerState,
+    MediaType,
+    RepeatMode,
+    async_process_play_media_url,
 )
-from homeassistant.components.media_player.const import (
-    MEDIA_CLASS_DIRECTORY,
-    MEDIA_CLASS_TRACK,
-    MEDIA_TYPE_MUSIC,
-    REPEAT_MODE_OFF,
-    SUPPORT_BROWSE_MEDIA,
-    SUPPORT_GROUPING,
-    SUPPORT_NEXT_TRACK,
-    SUPPORT_PAUSE,
-    SUPPORT_PLAY,
-    SUPPORT_PLAY_MEDIA,
-    SUPPORT_PREVIOUS_TRACK,
-    SUPPORT_REPEAT_SET,
-    SUPPORT_SELECT_SOUND_MODE,
-    SUPPORT_SELECT_SOURCE,
-    SUPPORT_SHUFFLE_SET,
-    SUPPORT_STOP,
-    SUPPORT_TURN_OFF,
-    SUPPORT_TURN_ON,
-    SUPPORT_VOLUME_MUTE,
-    SUPPORT_VOLUME_SET,
-    SUPPORT_VOLUME_STEP,
-)
-from homeassistant.config_entries import SOURCE_IMPORT, ConfigEntry
-from homeassistant.const import (
-    CONF_HOST,
-    CONF_PORT,
-    STATE_IDLE,
-    STATE_OFF,
-    STATE_PAUSED,
-    STATE_PLAYING,
-)
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.exceptions import HomeAssistantError
-import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.entity import Entity
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-from homeassistant.helpers.typing import DiscoveryInfoType
 from homeassistant.util import uuid
 
 from . import MusicCastDataUpdateCoordinator, MusicCastDeviceEntity
@@ -59,7 +33,6 @@ from .const import (
     DEFAULT_ZONE,
     DOMAIN,
     HA_REPEAT_MODE_TO_MC_MAPPING,
-    INTERVAL_SECONDS,
     MC_REPEAT_MODE_TO_HA_MAPPING,
     MEDIA_CLASS_MAPPING,
     NULL_GROUP,
@@ -68,50 +41,14 @@ from .const import (
 _LOGGER = logging.getLogger(__name__)
 
 MUSIC_PLAYER_BASE_SUPPORT = (
-    SUPPORT_SHUFFLE_SET
-    | SUPPORT_REPEAT_SET
-    | SUPPORT_SELECT_SOUND_MODE
-    | SUPPORT_SELECT_SOURCE
-    | SUPPORT_GROUPING
-    | SUPPORT_PLAY_MEDIA
+    MediaPlayerEntityFeature.SHUFFLE_SET
+    | MediaPlayerEntityFeature.REPEAT_SET
+    | MediaPlayerEntityFeature.SELECT_SOUND_MODE
+    | MediaPlayerEntityFeature.SELECT_SOURCE
+    | MediaPlayerEntityFeature.GROUPING
+    | MediaPlayerEntityFeature.PLAY_MEDIA
 )
 
-PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
-    {
-        vol.Required(CONF_HOST): cv.string,
-        vol.Optional(CONF_PORT, default=5000): cv.port,
-        vol.Optional(INTERVAL_SECONDS, default=0): cv.positive_int,
-    }
-)
-
-
-async def async_setup_platform(
-    hass: HomeAssistant,
-    config,
-    async_add_devices: AddEntitiesCallback,
-    discovery_info: DiscoveryInfoType | None = None,
-) -> None:
-    """Import legacy configurations."""
-
-    if hass.config_entries.async_entries(DOMAIN) and config[CONF_HOST] not in [
-        entry.data[CONF_HOST] for entry in hass.config_entries.async_entries(DOMAIN)
-    ]:
-        _LOGGER.error(
-            "Configuration in configuration.yaml is not supported anymore. "
-            "Please add this device using the config flow: %s",
-            config[CONF_HOST],
-        )
-    else:
-        _LOGGER.warning(
-            "Configuration in configuration.yaml is deprecated. Use the config flow instead"
-        )
-
-        hass.async_create_task(
-            hass.config_entries.flow.async_init(
-                DOMAIN, context={"source": SOURCE_IMPORT}, data=config
-            )
-        )
-
 
 async def async_setup_entry(
     hass: HomeAssistant,
@@ -138,9 +75,12 @@ async def async_setup_entry(
 class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
     """The musiccast media player."""
 
+    _attr_media_content_type = MediaType.MUSIC
+    _attr_should_poll = False
+
     def __init__(self, zone_id, name, entry_id, coordinator):
         """Initialize the musiccast device."""
-        self._player_state = STATE_PLAYING
+        self._player_state = MediaPlayerState.PLAYING
         self._volume_muted = False
         self._shuffle = False
         self._zone_id = zone_id
@@ -155,34 +95,28 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
         self._volume_max = self.coordinator.data.zones[self._zone_id].max_volume
 
         self._cur_track = 0
-        self._repeat = REPEAT_MODE_OFF
+        self._repeat = RepeatMode.OFF
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Run when this Entity has been added to HA."""
         await super().async_added_to_hass()
         self.coordinator.entities.append(self)
         # Sensors should also register callbacks to HA when their state changes
-        self.coordinator.musiccast.register_callback(self.async_write_ha_state)
         self.coordinator.musiccast.register_group_update_callback(
             self.update_all_mc_entities
         )
-        self.coordinator.async_add_listener(self.async_schedule_check_client_list)
+        self.async_on_remove(
+            self.coordinator.async_add_listener(self.async_schedule_check_client_list)
+        )
 
-    async def async_will_remove_from_hass(self):
+    async def async_will_remove_from_hass(self) -> None:
         """Entity being removed from hass."""
         await super().async_will_remove_from_hass()
         self.coordinator.entities.remove(self)
         # The opposite of async_added_to_hass. Remove any registered call backs here.
-        self.coordinator.musiccast.remove_callback(self.async_write_ha_state)
         self.coordinator.musiccast.remove_group_update_callback(
             self.update_all_mc_entities
         )
-        self.coordinator.async_remove_listener(self.async_schedule_check_client_list)
-
-    @property
-    def should_poll(self):
-        """Push an update after each command."""
-        return False
 
     @property
     def ip_address(self):
@@ -210,21 +144,16 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
         """Return the content ID of current playing media."""
         return None
 
-    @property
-    def media_content_type(self):
-        """Return the content type of current playing media."""
-        return MEDIA_TYPE_MUSIC
-
     @property
     def state(self):
         """Return the state of the player."""
         if self.coordinator.data.zones[self._zone_id].power == "on":
             if self._is_netusb and self.coordinator.data.netusb_playback == "pause":
-                return STATE_PAUSED
+                return MediaPlayerState.PAUSED
             if self._is_netusb and self.coordinator.data.netusb_playback == "stop":
-                return STATE_IDLE
-            return STATE_PLAYING
-        return STATE_OFF
+                return MediaPlayerState.IDLE
+            return MediaPlayerState.PLAYING
+        return MediaPlayerState.OFF
 
     @property
     def volume_level(self):
@@ -268,36 +197,36 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
         """Return the unique ID for this media_player."""
         return f"{self.coordinator.data.device_id}_{self._zone_id}"
 
-    async def async_turn_on(self):
+    async def async_turn_on(self) -> None:
         """Turn the media player on."""
         await self.coordinator.musiccast.turn_on(self._zone_id)
         self.async_write_ha_state()
 
-    async def async_turn_off(self):
+    async def async_turn_off(self) -> None:
         """Turn the media player off."""
         await self.coordinator.musiccast.turn_off(self._zone_id)
         self.async_write_ha_state()
 
-    async def async_mute_volume(self, mute):
+    async def async_mute_volume(self, mute: bool) -> None:
         """Mute the volume."""
 
         await self.coordinator.musiccast.mute_volume(self._zone_id, mute)
         self.async_write_ha_state()
 
-    async def async_set_volume_level(self, volume):
+    async def async_set_volume_level(self, volume: float) -> None:
         """Set the volume level, range 0..1."""
         await self.coordinator.musiccast.set_volume_level(self._zone_id, volume)
         self.async_write_ha_state()
 
-    async def async_volume_up(self):
+    async def async_volume_up(self) -> None:
         """Turn volume up for media player."""
         await self.coordinator.musiccast.volume_up(self._zone_id)
 
-    async def async_volume_down(self):
+    async def async_volume_down(self) -> None:
         """Turn volume down for media player."""
         await self.coordinator.musiccast.volume_down(self._zone_id)
 
-    async def async_media_play(self):
+    async def async_media_play(self) -> None:
         """Send play command."""
         if self._is_netusb:
             await self.coordinator.musiccast.netusb_play()
@@ -306,7 +235,7 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
                 "Service play is not supported for non NetUSB sources."
             )
 
-    async def async_media_pause(self):
+    async def async_media_pause(self) -> None:
         """Send pause command."""
         if self._is_netusb:
             await self.coordinator.musiccast.netusb_pause()
@@ -315,7 +244,7 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
                 "Service pause is not supported for non NetUSB sources."
             )
 
-    async def async_media_stop(self):
+    async def async_media_stop(self) -> None:
         """Send stop command."""
         if self._is_netusb:
             await self.coordinator.musiccast.netusb_stop()
@@ -324,7 +253,7 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
                 "Service stop is not supported for non NetUSB sources."
             )
 
-    async def async_set_shuffle(self, shuffle):
+    async def async_set_shuffle(self, shuffle: bool) -> None:
         """Enable/disable shuffle mode."""
         if self._is_netusb:
             await self.coordinator.musiccast.netusb_shuffle(shuffle)
@@ -333,9 +262,17 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
                 "Service shuffle is not supported for non NetUSB sources."
             )
 
-    async def async_play_media(self, media_type: str, media_id: str, **kwargs) -> None:
+    async def async_play_media(
+        self, media_type: str, media_id: str, **kwargs: Any
+    ) -> None:
         """Play media."""
-        if self.state == STATE_OFF:
+        if media_source.is_media_source_id(media_id):
+            play_item = await media_source.async_resolve_media(
+                self.hass, media_id, self.entity_id
+            )
+            media_id = play_item.url
+
+        if self.state == MediaPlayerState.OFF:
             await self.async_turn_on()
 
         if media_id:
@@ -355,7 +292,9 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
                 )
                 return
 
-            if parts[0] == "http":
+            if parts[0] in ("http", "https") or media_id.startswith("/"):
+                media_id = async_process_play_media_url(self.hass, media_id)
+
                 await self.coordinator.musiccast.play_url_media(
                     self._zone_id, media_id, "HomeAssistant"
                 )
@@ -367,7 +306,16 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
 
     async def async_browse_media(self, media_content_type=None, media_content_id=None):
         """Implement the websocket media browsing helper."""
-        if self.state == STATE_OFF:
+        if media_content_id and media_source.is_media_source_id(media_content_id):
+            return await media_source.async_browse_media(
+                self.hass,
+                media_content_id,
+                content_filter=lambda item: item.media_content_type.startswith(
+                    "audio/"
+                ),
+            )
+
+        if self.state == MediaPlayerState.OFF:
             raise HomeAssistantError(
                 "The device has to be turned on to be able to browse media."
             )
@@ -377,16 +325,18 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
             media_content_provider = await MusicCastMediaContent.browse_media(
                 self.coordinator.musiccast, self._zone_id, media_content_path, 24
             )
+            add_media_source = False
 
         else:
             media_content_provider = MusicCastMediaContent.categories(
                 self.coordinator.musiccast, self._zone_id
             )
+            add_media_source = True
 
         def get_content_type(item):
             if item.can_play:
-                return MEDIA_CLASS_TRACK
-            return MEDIA_CLASS_DIRECTORY
+                return MediaClass.TRACK
+            return MediaClass.DIRECTORY
 
         children = [
             BrowseMedia(
@@ -401,6 +351,21 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
             for child in media_content_provider.children
         ]
 
+        if add_media_source:
+            with contextlib.suppress(media_source.BrowseError):
+                item = await media_source.async_browse_media(
+                    self.hass,
+                    None,
+                    content_filter=lambda item: item.media_content_type.startswith(
+                        "audio/"
+                    ),
+                )
+                # If domain is None, it's overview of available sources
+                if item.domain is None:
+                    children.extend(item.children)
+                else:
+                    children.append(item)
+
         overview = BrowseMedia(
             title=media_content_provider.title,
             media_class=MEDIA_CLASS_MAPPING.get(media_content_provider.content_type),
@@ -413,7 +378,7 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
 
         return overview
 
-    async def async_select_sound_mode(self, sound_mode):
+    async def async_select_sound_mode(self, sound_mode: str) -> None:
         """Select sound mode."""
         await self.coordinator.musiccast.select_sound_mode(self._zone_id, sound_mode)
 
@@ -455,7 +420,7 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
         return (
             MC_REPEAT_MODE_TO_HA_MAPPING.get(self.coordinator.data.netusb_repeat)
             if self._is_netusb
-            else REPEAT_MODE_OFF
+            else RepeatMode.OFF
         )
 
     @property
@@ -465,27 +430,32 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
         zone = self.coordinator.data.zones[self._zone_id]
 
         if ZoneFeature.POWER in zone.features:
-            supported_features |= SUPPORT_TURN_ON | SUPPORT_TURN_OFF
+            supported_features |= (
+                MediaPlayerEntityFeature.TURN_ON | MediaPlayerEntityFeature.TURN_OFF
+            )
         if ZoneFeature.VOLUME in zone.features:
-            supported_features |= SUPPORT_VOLUME_SET | SUPPORT_VOLUME_STEP
+            supported_features |= (
+                MediaPlayerEntityFeature.VOLUME_SET
+                | MediaPlayerEntityFeature.VOLUME_STEP
+            )
         if ZoneFeature.MUTE in zone.features:
-            supported_features |= SUPPORT_VOLUME_MUTE
+            supported_features |= MediaPlayerEntityFeature.VOLUME_MUTE
 
         if self._is_netusb or self._is_tuner:
-            supported_features |= SUPPORT_PREVIOUS_TRACK
-            supported_features |= SUPPORT_NEXT_TRACK
+            supported_features |= MediaPlayerEntityFeature.PREVIOUS_TRACK
+            supported_features |= MediaPlayerEntityFeature.NEXT_TRACK
 
         if self._is_netusb:
-            supported_features |= SUPPORT_PAUSE
-            supported_features |= SUPPORT_PLAY
-            supported_features |= SUPPORT_STOP
+            supported_features |= MediaPlayerEntityFeature.PAUSE
+            supported_features |= MediaPlayerEntityFeature.PLAY
+            supported_features |= MediaPlayerEntityFeature.STOP
 
-        if self.state != STATE_OFF:
-            supported_features |= SUPPORT_BROWSE_MEDIA
+        if self.state != MediaPlayerState.OFF:
+            supported_features |= MediaPlayerEntityFeature.BROWSE_MEDIA
 
         return supported_features
 
-    async def async_media_previous_track(self):
+    async def async_media_previous_track(self) -> None:
         """Send previous track command."""
         if self._is_netusb:
             await self.coordinator.musiccast.netusb_previous_track()
@@ -496,7 +466,7 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
                 "Service previous track is not supported for non NetUSB or Tuner sources."
             )
 
-    async def async_media_next_track(self):
+    async def async_media_next_track(self) -> None:
         """Send next track command."""
         if self._is_netusb:
             await self.coordinator.musiccast.netusb_next_track()
@@ -507,7 +477,7 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
                 "Service next track is not supported for non NetUSB or Tuner sources."
             )
 
-    async def async_set_repeat(self, repeat):
+    async def async_set_repeat(self, repeat: RepeatMode) -> None:
         """Enable/disable repeat mode."""
         if self._is_netusb:
             await self.coordinator.musiccast.netusb_repeat(
@@ -518,7 +488,7 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
                 "Service set repeat is not supported for non NetUSB sources."
             )
 
-    async def async_select_source(self, source):
+    async def async_select_source(self, source: str) -> None:
         """Select input source."""
         await self.coordinator.musiccast.select_source(self._zone_id, source)
 
@@ -709,7 +679,7 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
 
     # Services
 
-    async def async_join_players(self, group_members):
+    async def async_join_players(self, group_members: list[str]) -> None:
         """Add all clients given in entities to the group of the server.
 
         Creates a new group if necessary. Used for join service.
@@ -726,7 +696,7 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
             if entity.entity_id in group_members
         ]
 
-        if self.state == STATE_OFF:
+        if self.state == MediaPlayerState.OFF:
             await self.async_turn_on()
 
         if not self.is_server and self.musiccast_zone_entity.is_server:
@@ -776,7 +746,7 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
 
         await self.update_all_mc_entities(True)
 
-    async def async_unjoin_player(self):
+    async def async_unjoin_player(self) -> None:
         """Leave the group.
 
         Stops the distribution if device is server. Used for unjoin service.
@@ -800,7 +770,7 @@ class MusicCastMediaPlayer(MusicCastDeviceEntity, MediaPlayerEntity):
         """
         # If we should join the group, which is served by the main zone, we can simply select main_sync as input.
         _LOGGER.debug("%s called service client join", self.entity_id)
-        if self.state == STATE_OFF:
+        if self.state == MediaPlayerState.OFF:
             await self.async_turn_on()
         if self.ip_address == server.ip_address:
             if server.zone == DEFAULT_ZONE:
