@@ -7,8 +7,8 @@ from .backports.enum import StrEnum
 
 APPLICATION_NAME: Final = "HomeAssistant"
 MAJOR_VERSION: Final = 2022
-MINOR_VERSION: Final = 10
-PATCH_VERSION: Final = "4"
+MINOR_VERSION: Final = 11
+PATCH_VERSION: Final = "5"
 __short_version__: Final = f"{MAJOR_VERSION}.{MINOR_VERSION}"
 __version__: Final = f"{__short_version__}.{PATCH_VERSION}"
 REQUIRED_PYTHON_VER: Final[tuple[int, int, int]] = (3, 9, 0)
@@ -396,7 +396,9 @@ ATTR_ICON: Final = "icon"
 ATTR_UNIT_OF_MEASUREMENT: Final = "unit_of_measurement"
 
 CONF_UNIT_SYSTEM_METRIC: Final = "metric"
+"""Deprecated: please use a local constant."""
 CONF_UNIT_SYSTEM_IMPERIAL: Final = "imperial"
+"""Deprecated: please use a local constant."""
 
 # Electrical attributes
 ATTR_VOLTAGE: Final = "voltage"
@@ -476,18 +478,43 @@ ATTR_PERSONS: Final = "persons"
 # Apparent power units
 POWER_VOLT_AMPERE: Final = "VA"
 
+
 # Power units
+class UnitOfPower(StrEnum):
+    """Power units."""
+
+    WATT = "W"
+    KILO_WATT = "kW"
+    BTU_PER_HOUR = "BTU/h"
+
+
 POWER_WATT: Final = "W"
+"""Deprecated: please use UnitOfPower.WATT."""
 POWER_KILO_WATT: Final = "kW"
+"""Deprecated: please use UnitOfPower.KILO_WATT."""
 POWER_BTU_PER_HOUR: Final = "BTU/h"
+"""Deprecated: please use UnitOfPower.BTU_PER_HOUR."""
 
 # Reactive power units
 POWER_VOLT_AMPERE_REACTIVE: Final = "var"
 
+
 # Energy units
-ENERGY_WATT_HOUR: Final = "Wh"
+class UnitOfEnergy(StrEnum):
+    """Energy units."""
+
+    GIGA_JOULE = "GJ"
+    KILO_WATT_HOUR = "kWh"
+    MEGA_WATT_HOUR = "MWh"
+    WATT_HOUR = "Wh"
+
+
 ENERGY_KILO_WATT_HOUR: Final = "kWh"
+"""Deprecated: please use UnitOfEnergy.KILO_WATT_HOUR."""
 ENERGY_MEGA_WATT_HOUR: Final = "MWh"
+"""Deprecated: please use UnitOfEnergy.MEGA_WATT_HOUR."""
+ENERGY_WATT_HOUR: Final = "Wh"
+"""Deprecated: please use UnitOfEnergy.WATT_HOUR."""
 
 # Electric_current units
 ELECTRIC_CURRENT_MILLIAMPERE: Final = "mA"
@@ -505,10 +532,22 @@ CURRENCY_EURO: Final = "€"
 CURRENCY_DOLLAR: Final = "$"
 CURRENCY_CENT: Final = "¢"
 
+
 # Temperature units
+class UnitOfTemperature(StrEnum):
+    """Temperature units."""
+
+    CELSIUS = "°C"
+    FAHRENHEIT = "°F"
+    KELVIN = "K"
+
+
 TEMP_CELSIUS: Final = "°C"
+"""Deprecated: please use UnitOfTemperature.CELSIUS"""
 TEMP_FAHRENHEIT: Final = "°F"
+"""Deprecated: please use UnitOfTemperature.FAHRENHEIT"""
 TEMP_KELVIN: Final = "K"
+"""Deprecated: please use UnitOfTemperature.KELVIN"""
 
 # Time units
 TIME_MICROSECONDS: Final = "μs"
@@ -521,16 +560,37 @@ TIME_WEEKS: Final = "w"
 TIME_MONTHS: Final = "m"
 TIME_YEARS: Final = "y"
 
-# Length units
-LENGTH_MILLIMETERS: Final = "mm"
-LENGTH_CENTIMETERS: Final = "cm"
-LENGTH_METERS: Final = "m"
-LENGTH_KILOMETERS: Final = "km"
 
+# Length units
+class UnitOfLength(StrEnum):
+    """Length units."""
+
+    MILLIMETERS = "mm"
+    CENTIMETERS = "cm"
+    METERS = "m"
+    KILOMETERS = "km"
+    INCHES = "in"
+    FEET = "ft"
+    YARDS = "yd"
+    MILES = "mi"
+
+
+LENGTH_MILLIMETERS: Final = "mm"
+"""Deprecated: please use UnitOfLength.MILLIMETERS."""
+LENGTH_CENTIMETERS: Final = "cm"
+"""Deprecated: please use UnitOfLength.CENTIMETERS."""
+LENGTH_METERS: Final = "m"
+"""Deprecated: please use UnitOfLength.METERS."""
+LENGTH_KILOMETERS: Final = "km"
+"""Deprecated: please use UnitOfLength.KILOMETERS."""
 LENGTH_INCHES: Final = "in"
+"""Deprecated: please use UnitOfLength.INCHES."""
 LENGTH_FEET: Final = "ft"
+"""Deprecated: please use UnitOfLength.FEET."""
 LENGTH_YARD: Final = "yd"
+"""Deprecated: please use UnitOfLength.YARDS."""
 LENGTH_MILES: Final = "mi"
+"""Deprecated: please use UnitOfLength.MILES."""
 
 # Frequency units
 FREQUENCY_HERTZ: Final = "Hz"
@@ -538,32 +598,77 @@ FREQUENCY_KILOHERTZ: Final = "kHz"
 FREQUENCY_MEGAHERTZ: Final = "MHz"
 FREQUENCY_GIGAHERTZ: Final = "GHz"
 
+
 # Pressure units
+class UnitOfPressure(StrEnum):
+    """Pressure units."""
+
+    PA = "Pa"
+    HPA = "hPa"
+    KPA = "kPa"
+    BAR = "bar"
+    CBAR = "cbar"
+    MBAR = "mbar"
+    MMHG = "mmHg"
+    INHG = "inHg"
+    PSI = "psi"
+
+
 PRESSURE_PA: Final = "Pa"
+"""Deprecated: please use UnitOfPressure.PA"""
 PRESSURE_HPA: Final = "hPa"
+"""Deprecated: please use UnitOfPressure.HPA"""
 PRESSURE_KPA: Final = "kPa"
+"""Deprecated: please use UnitOfPressure.KPA"""
 PRESSURE_BAR: Final = "bar"
+"""Deprecated: please use UnitOfPressure.BAR"""
 PRESSURE_CBAR: Final = "cbar"
+"""Deprecated: please use UnitOfPressure.CBAR"""
 PRESSURE_MBAR: Final = "mbar"
+"""Deprecated: please use UnitOfPressure.MBAR"""
 PRESSURE_MMHG: Final = "mmHg"
+"""Deprecated: please use UnitOfPressure.MMHG"""
 PRESSURE_INHG: Final = "inHg"
+"""Deprecated: please use UnitOfPressure.INHG"""
 PRESSURE_PSI: Final = "psi"
+"""Deprecated: please use UnitOfPressure.PSI"""
 
 # Sound pressure units
 SOUND_PRESSURE_DB: Final = "dB"
 SOUND_PRESSURE_WEIGHTED_DBA: Final = "dBa"
 
+
 # Volume units
+class UnitOfVolume(StrEnum):
+    """Volume units."""
+
+    CUBIC_FEET = "ft³"
+    CUBIC_METERS = "m³"
+    LITERS = "L"
+    MILLILITERS = "mL"
+    GALLONS = "gal"
+    """Assumed to be US gallons in conversion utilities.
+
+    British/Imperial gallons are not yet supported"""
+    FLUID_OUNCES = "fl. oz."
+    """Assumed to be US fluid ounces in conversion utilities.
+
+    British/Imperial fluid ounces are not yet supported"""
+
+
 VOLUME_LITERS: Final = "L"
+"""Deprecated: please use UnitOfVolume.LITERS"""
 VOLUME_MILLILITERS: Final = "mL"
+"""Deprecated: please use UnitOfVolume.MILLILITERS"""
 VOLUME_CUBIC_METERS: Final = "m³"
+"""Deprecated: please use UnitOfVolume.CUBIC_METERS"""
 VOLUME_CUBIC_FEET: Final = "ft³"
+"""Deprecated: please use UnitOfVolume.CUBIC_FEET"""
 
 VOLUME_GALLONS: Final = "gal"
-"""US gallon (British gallon is not yet supported)"""
-
+"""Deprecated: please use UnitOfVolume.GALLONS"""
 VOLUME_FLUID_OUNCE: Final = "fl. oz."
-"""US fluid ounce (British fluid ounce is not yet supported)"""
+"""Deprecated: please use UnitOfVolume.FLUID_OUNCES"""
 
 # Volume Flow Rate units
 VOLUME_FLOW_RATE_CUBIC_METERS_PER_HOUR: Final = "m³/h"
@@ -572,14 +677,31 @@ VOLUME_FLOW_RATE_CUBIC_FEET_PER_MINUTE: Final = "ft³/m"
 # Area units
 AREA_SQUARE_METERS: Final = "m²"
 
-# Mass units
-MASS_GRAMS: Final = "g"
-MASS_KILOGRAMS: Final = "kg"
-MASS_MILLIGRAMS: Final = "mg"
-MASS_MICROGRAMS: Final = "µg"
 
+# Mass units
+class UnitOfMass(StrEnum):
+    """Mass units."""
+
+    GRAMS = "g"
+    KILOGRAMS = "kg"
+    MILLIGRAMS = "mg"
+    MICROGRAMS = "µg"
+    OUNCES = "oz"
+    POUNDS = "lb"
+
+
+MASS_GRAMS: Final = "g"
+"""Deprecated: please use UnitOfMass.GRAMS"""
+MASS_KILOGRAMS: Final = "kg"
+"""Deprecated: please use UnitOfMass.KILOGRAMS"""
+MASS_MILLIGRAMS: Final = "mg"
+"""Deprecated: please use UnitOfMass.MILLIGRAMS"""
+MASS_MICROGRAMS: Final = "µg"
+"""Deprecated: please use UnitOfMass.MICROGRAMS"""
 MASS_OUNCES: Final = "oz"
+"""Deprecated: please use UnitOfMass.OUNCES"""
 MASS_POUNDS: Final = "lb"
+"""Deprecated: please use UnitOfMass.POUNDS"""
 
 # Conductivity units
 CONDUCTIVITY: Final = "µS/cm"
@@ -600,10 +722,38 @@ REVOLUTIONS_PER_MINUTE: Final = "rpm"
 IRRADIATION_WATTS_PER_SQUARE_METER: Final = "W/m²"
 IRRADIATION_BTUS_PER_HOUR_SQUARE_FOOT: Final = "BTU/(h×ft²)"
 
+
+class UnitOfVolumetricFlux(StrEnum):
+    """Volumetric flux, commonly used for precipitation intensity.
+
+    The derivation of these units is a volume of rain amassing in a container
+    with constant cross section in a given time
+    """
+
+    INCHES_PER_DAY = "in/d"
+    """Derived from in³/(in².d)"""
+
+    INCHES_PER_HOUR = "in/h"
+    """Derived from in³/(in².h)"""
+
+    MILLIMETERS_PER_DAY = "mm/d"
+    """Derived from mm³/(mm².d)"""
+
+    MILLIMETERS_PER_HOUR = "mm/h"
+    """Derived from mm³/(mm².h)"""
+
+
 # Precipitation units
-PRECIPITATION_MILLIMETERS_PER_HOUR: Final = "mm/h"
+# The derivation of these units is a volume of rain amassing in a container
+# with constant cross section
 PRECIPITATION_INCHES: Final = "in"
+PRECIPITATION_MILLIMETERS: Final = "mm"
+
+PRECIPITATION_MILLIMETERS_PER_HOUR: Final = "mm/h"
+"""Deprecated: please use UnitOfVolumetricFlux.MILLIMETERS_PER_HOUR"""
+
 PRECIPITATION_INCHES_PER_HOUR: Final = "in/h"
+"""Deprecated: please use UnitOfVolumetricFlux.INCHES_PER_HOUR"""
 
 # Concentration units
 CONCENTRATION_MICROGRAMS_PER_CUBIC_METER: Final = "µg/m³"
@@ -613,15 +763,38 @@ CONCENTRATION_PARTS_PER_CUBIC_METER: Final = "p/m³"
 CONCENTRATION_PARTS_PER_MILLION: Final = "ppm"
 CONCENTRATION_PARTS_PER_BILLION: Final = "ppb"
 
+
 # Speed units
-SPEED_MILLIMETERS_PER_DAY: Final = "mm/d"
+class UnitOfSpeed(StrEnum):
+    """Speed units."""
+
+    FEET_PER_SECOND = "ft/s"
+    METERS_PER_SECOND = "m/s"
+    KILOMETERS_PER_HOUR = "km/h"
+    KNOTS = "kn"
+    MILES_PER_HOUR = "mph"
+
+
 SPEED_FEET_PER_SECOND: Final = "ft/s"
-SPEED_INCHES_PER_DAY: Final = "in/d"
+"""Deprecated: please use UnitOfSpeed.FEET_PER_SECOND"""
 SPEED_METERS_PER_SECOND: Final = "m/s"
-SPEED_INCHES_PER_HOUR: Final = "in/h"
+"""Deprecated: please use UnitOfSpeed.METERS_PER_SECOND"""
 SPEED_KILOMETERS_PER_HOUR: Final = "km/h"
+"""Deprecated: please use UnitOfSpeed.KILOMETERS_PER_HOUR"""
 SPEED_KNOTS: Final = "kn"
+"""Deprecated: please use UnitOfSpeed.KNOTS"""
 SPEED_MILES_PER_HOUR: Final = "mph"
+"""Deprecated: please use UnitOfSpeed.MILES_PER_HOUR"""
+
+SPEED_MILLIMETERS_PER_DAY: Final = "mm/d"
+"""Deprecated: please use UnitOfVolumetricFlux.MILLIMETERS_PER_DAY"""
+
+SPEED_INCHES_PER_DAY: Final = "in/d"
+"""Deprecated: please use UnitOfVolumetricFlux.INCHES_PER_DAY"""
+
+SPEED_INCHES_PER_HOUR: Final = "in/h"
+"""Deprecated: please use UnitOfVolumetricFlux.INCHES_PER_HOUR"""
+
 
 # Signal_strength units
 SIGNAL_STRENGTH_DECIBELS: Final = "dB"
@@ -786,4 +959,4 @@ CAST_APP_ID_HOMEASSISTANT_LOVELACE: Final = "A078F6B0"
 # User used by Supervisor
 HASSIO_USER_NAME = "Supervisor"
 
-SIGNAL_BOOTSTRAP_INTEGRATONS = "bootstrap_integrations"
+SIGNAL_BOOTSTRAP_INTEGRATIONS = "bootstrap_integrations"
