@@ -7,7 +7,7 @@ from datetime import timedelta
 from itertools import chain
 import logging
 from types import ModuleType
-from typing import Any
+from typing import Any, Generic, TypeVar
 
 import voluptuous as vol
 
@@ -30,11 +30,14 @@ from .typing import ConfigType, DiscoveryInfoType
 DEFAULT_SCAN_INTERVAL = timedelta(seconds=15)
 DATA_INSTANCES = "entity_components"
 
+_EntityT = TypeVar("_EntityT", bound=entity.Entity)
+
 
 @bind_hass
 async def async_update_entity(hass: HomeAssistant, entity_id: str) -> None:
     """Trigger an update for an entity."""
     domain = entity_id.split(".", 1)[0]
+    entity_comp: EntityComponent[entity.Entity] | None
     entity_comp = hass.data.get(DATA_INSTANCES, {}).get(domain)
 
     if entity_comp is None:
@@ -52,7 +55,7 @@ async def async_update_entity(hass: HomeAssistant, entity_id: str) -> None:
     await entity_obj.async_update_ha_state(True)
 
 
-class EntityComponent:
+class EntityComponent(Generic[_EntityT]):
     """The EntityComponent manages platforms that manages entities.
 
     This class has the following responsibilities:
@@ -86,18 +89,19 @@ class EntityComponent:
         hass.data.setdefault(DATA_INSTANCES, {})[domain] = self
 
     @property
-    def entities(self) -> Iterable[entity.Entity]:
+    def entities(self) -> Iterable[_EntityT]:
         """Return an iterable that returns all entities."""
         return chain.from_iterable(
-            platform.entities.values() for platform in self._platforms.values()
+            platform.entities.values()  # type: ignore[misc]
+            for platform in self._platforms.values()
         )
 
-    def get_entity(self, entity_id: str) -> entity.Entity | None:
+    def get_entity(self, entity_id: str) -> _EntityT | None:
         """Get an entity."""
         for platform in self._platforms.values():
             entity_obj = platform.entities.get(entity_id)
             if entity_obj is not None:
-                return entity_obj
+                return entity_obj  # type: ignore[return-value]
         return None
 
     def setup(self, config: ConfigType) -> None:
@@ -176,7 +180,7 @@ class EntityComponent:
 
     async def async_extract_from_service(
         self, service_call: ServiceCall, expand_group: bool = True
-    ) -> list[entity.Entity]:
+    ) -> list[_EntityT]:
         """Extract all known and available entities from a service call.
 
         Will return an empty list if entities specified but unknown.
@@ -201,8 +205,8 @@ class EntityComponent:
 
         async def handle_service(call: ServiceCall) -> None:
             """Handle the service."""
-            await self.hass.helpers.service.entity_service_call(
-                self._platforms.values(), func, call, required_features
+            await service.entity_service_call(
+                self.hass, self._platforms.values(), func, call, required_features
             )
 
         self.hass.services.async_register(self.domain, name, handle_service, schema)
